@@ -3,15 +3,19 @@ module GHC.Util.ApiAnnotation (
     comment, commentText, isCommentMultiline
   , pragmas, flags, languagePragmas
   , mkFlags, mkLanguagePragmas
+  , extensions
 ) where
 
-import ApiAnnotation
-import SrcLoc
+import GHC.LanguageExtensions.Type (Extension)
+import GHC.Parser.Annotation
+import GHC.Types.SrcLoc
+
+import Language.Haskell.GhclibParserEx.GHC.Driver.Session
 
 import Control.Applicative
-import qualified Data.Map.Strict as Map
-import Data.Maybe
 import Data.List.Extra
+import Data.Maybe
+import qualified Data.Set as Set
 
 trimCommentStart :: String -> String
 trimCommentStart s
@@ -56,12 +60,20 @@ pragmas anns =
   -- encountered in the source file with the last at the head of the
   -- list (makes sense when you think about it).
   reverse
-    [ (c, s) |
-        c@(L _ (AnnBlockComment comm)) <- fromMaybe [] $ Map.lookup noSrcSpan (snd anns)
+    [ (realToLoc c, s) |
+        c@(L _ (AnnBlockComment comm)) <- apiAnnRogueComments anns
       , let body = trimCommentDelims comm
       , Just rest <- [stripSuffix "#" =<< stripPrefix "#" body]
       , let s = trim rest
     ]
+   where
+     realToLoc :: RealLocated a -> Located a
+     realToLoc (L r x) = L (RealSrcSpan r Nothing) x
+
+-- All the extensions defined to be used.
+extensions :: ApiAnns -> Set.Set Extension
+extensions = Set.fromList . mapMaybe readExtension .
+    concatMap snd . languagePragmas . pragmas
 
 -- Utility for a case insensitive prefix strip.
 stripPrefixCI :: String -> String -> Maybe String
