@@ -7,13 +7,12 @@ module GHC.Util.FreeVars (
     Vars (..), FreeVars(..) , AllVars (..)
   ) where
 
-import RdrName
-import GHC.Hs.Types
-import OccName
-import Name
+import GHC.Types.Name.Reader
+import GHC.Types.Name.Occurrence
+import GHC.Types.Name
 import GHC.Hs
-import SrcLoc
-import Bag (bagToList)
+import GHC.Types.SrcLoc
+import GHC.Data.Bag (bagToList)
 
 import Data.Generics.Uniplate.DataOnly
 import Data.Monoid
@@ -98,7 +97,7 @@ unqualNames _ = []
 
 instance FreeVars (LHsExpr GhcPs) where
   freeVars (L _ (HsVar _ x)) = Set.fromList $ unqualNames x -- Variable.
-  freeVars (L _ (HsUnboundVar _ x)) = Set.fromList [unboundVarOcc x] -- Unbound variable; also used for "holes".
+  freeVars (L _ (HsUnboundVar _ x)) = Set.fromList [x] -- Unbound variable; also used for "holes".
   freeVars (L _ (HsLam _ mg)) = free (allVars mg) -- Lambda abstraction. Currently always a single match.
   freeVars (L _ (HsLamCase _ MG{mg_alts=(L _ ms)})) = free (allVars ms) -- Lambda case
   freeVars (L _ (HsCase _ of_ MG{mg_alts=(L _ ms)})) = freeVars of_ ^+ free (allVars ms) -- Case expr.
@@ -107,6 +106,8 @@ instance FreeVars (LHsExpr GhcPs) where
   freeVars (L _ (RecordCon _ _ (HsRecFields flds _))) = Set.unions $ map freeVars flds -- Record construction.
   freeVars (L _ (RecordUpd _ e flds)) = Set.unions $ freeVars e : map freeVars flds -- Record update.
   freeVars (L _ (HsMultiIf _ grhss)) = free (allVars grhss) -- Multi-way if.
+  freeVars (L _ (HsBracket _ (ExpBr _ e))) = freeVars e
+  freeVars (L _ (HsBracket _ (VarBr _ _ v))) = Set.fromList [occName v]
 
   freeVars (L _ HsConLikeOut{}) = mempty -- After typechecker.
   freeVars (L _ HsRecFld{}) = mempty -- Variable pointing to a record selector.
@@ -116,7 +117,6 @@ instance FreeVars (LHsExpr GhcPs) where
   freeVars (L _ HsLit{}) = mempty -- Simple literal.
   freeVars (L _ HsRnBracketOut{}) = mempty -- Renamer produces these.
   freeVars (L _ HsTcBracketOut{}) = mempty -- Typechecker produces these.
-  freeVars (L _ HsWrap{}) = mempty -- Typechecker output.
 
   -- freeVars (e@(L _ HsAppType{})) = freeVars $ children e -- Visible type application e.g. f @ Int x y.
   -- freeVars (e@(L _ HsApp{})) = freeVars $ children e -- Application.
@@ -162,12 +162,11 @@ instance FreeVars (LHsRecUpdField GhcPs) where
 instance AllVars (Located (Pat GhcPs)) where
   allVars (L _ (VarPat _ (L _ x))) = Vars (Set.singleton $ rdrNameOcc x) Set.empty -- Variable pattern.
   allVars (L _ (AsPat _  n x)) = allVars (noLoc $ VarPat noExtField n :: LPat GhcPs) <> allVars x -- As pattern.
-  allVars (L _ (ConPatIn _ (RecCon (HsRecFields flds _)))) = allVars flds
+  allVars (L _ (ConPat _ _ (RecCon (HsRecFields flds _)))) = allVars flds
   allVars (L _ (NPlusKPat _ n _ _ _ _)) = allVars (noLoc $ VarPat noExtField n :: LPat GhcPs) -- n+k pattern.
   allVars (L _ (ViewPat _ e p)) = freeVars_ e <> allVars p -- View pattern.
 
   allVars (L _ WildPat{}) = mempty -- Wildcard pattern.
-  allVars (L _ ConPatOut{}) = mempty -- Renamer/typechecker.
   allVars (L _ LitPat{}) = mempty -- Literal pattern.
   allVars (L _ NPat{}) = mempty -- Natural pattern.
 
@@ -188,7 +187,7 @@ instance AllVars (LHsRecField GhcPs (Located (Pat GhcPs))) where
 
 instance AllVars (LStmt GhcPs (LHsExpr GhcPs)) where
   allVars (L _ (LastStmt _ expr _ _)) = freeVars_ expr -- The last stmt of a ListComp, MonadComp, DoExpr,MDoExpr.
-  allVars (L _ (BindStmt _ pat expr _ _)) = allVars pat <> freeVars_ expr -- A generator e.g. x <- [1, 2, 3].
+  allVars (L _ (BindStmt _ pat expr)) = allVars pat <> freeVars_ expr -- A generator e.g. x <- [1, 2, 3].
   allVars (L _ (BodyStmt _ expr _ _)) = freeVars_ expr -- A boolean guard e.g. even x.
   allVars (L _ (LetStmt _ binds)) = allVars binds -- A local declaration e.g. let y = x + 1
   allVars (L _ (TransStmt _ _ stmts _ using by _ _ fmap_)) = allVars stmts <> freeVars_ using <> maybe mempty freeVars_ by <> freeVars_ (noLoc fmap_ :: Located (HsExpr GhcPs)) -- Apply a function to a list of statements in order.
@@ -197,21 +196,15 @@ instance AllVars (LStmt GhcPs (LHsExpr GhcPs)) where
   allVars (L _ ApplicativeStmt{}) = mempty -- Generated by the renamer.
   allVars (L _ ParStmt{}) = mempty -- Parallel list thing. Come back to it.
 
-  allVars _ = mempty -- New ctor.
-
 instance AllVars (LHsLocalBinds GhcPs) where
   allVars (L _ (HsValBinds _ (ValBinds _ binds _))) = allVars (bagToList binds) -- Value bindings.
   allVars (L _ (HsIPBinds _ (IPBinds _ binds))) = allVars binds -- Implicit parameter bindings.
-
   allVars (L _ EmptyLocalBinds{}) =  mempty -- The case of no local bindings (signals the empty `let` or `where` clause).
-
-  allVars _ = mempty -- New ctor.
+  allVars _ = mempty -- extension points
 
 instance AllVars (LIPBind GhcPs) where
   allVars (L _ (IPBind _ _ e)) = freeVars_ e
 
-  allVars _ = mempty -- New ctor.
-
 instance AllVars (LHsBind GhcPs) where
   allVars (L _ FunBind{fun_id=n, fun_matches=MG{mg_alts=(L _ ms)}}) = allVars (noLoc $ VarPat noExtField n :: LPat GhcPs) <> allVars ms -- Function bindings and simple variable bindings e.g. f x = e, f !x = 3, f = e, !x = e, x `f` y = e
   allVars (L _ PatBind{pat_lhs=n, pat_rhs=grhss}) = allVars n <> allVars grhss -- Ctor patterns and some other interesting cases e.g. Just x = e, (x) = e, x :: Ty = e.
@@ -220,41 +213,30 @@ instance AllVars (LHsBind GhcPs) where
   allVars (L _ VarBind{}) = mempty -- Typechecker.
   allVars (L _ AbsBinds{}) = mempty -- Not sure but I think renamer.
 
-  allVars _ = mempty -- New ctor.
-
 instance AllVars (MatchGroup GhcPs (LHsExpr GhcPs)) where
   allVars (MG _ _alts@(L _ alts) _) = inVars (foldMap (allVars . m_pats) ms) (allVars (map m_grhss ms))
     where ms = map unLoc alts
-  allVars _ = mempty -- New ctor.
 
 instance AllVars (LMatch GhcPs (LHsExpr GhcPs)) where
   allVars (L _ (Match _ FunRhs {mc_fun=name} pats grhss)) = allVars (noLoc $ VarPat noExtField name :: LPat GhcPs) <> allVars pats <> allVars grhss -- A pattern matching on an argument of a function binding.
   allVars (L _ (Match _ (StmtCtxt ctxt) pats grhss)) = allVars ctxt <> allVars pats <> allVars grhss -- Pattern of a do-stmt, list comprehension, pattern guard etc.
   allVars (L _ (Match _ _ pats grhss)) = inVars (allVars pats) (allVars grhss) -- Everything else.
 
-  allVars _ = mempty -- New ctor.
-
-instance AllVars (HsStmtContext RdrName) where
+instance AllVars (HsStmtContext GhcPs) where
   allVars (PatGuard FunRhs{mc_fun=n}) = allVars (noLoc $ VarPat noExtField n :: LPat GhcPs)
   allVars ParStmtCtxt{} = mempty -- Come back to it.
   allVars TransStmtCtxt{}  = mempty -- Come back to it.
-
-  allVars _ = mempty -- Everything else (correct).
+  allVars _ = mempty
 
 instance AllVars (GRHSs GhcPs (LHsExpr GhcPs)) where
   allVars (GRHSs _ grhss binds) = inVars binds (mconcatMap allVars grhss)
 
-  allVars _ = mempty -- New ctor.
-
 instance AllVars (LGRHS GhcPs (LHsExpr GhcPs)) where
   allVars (L _ (GRHS _ guards expr)) = Vars (bound gs) (free gs ^+ (freeVars expr ^- bound gs)) where gs = allVars guards
 
-  allVars _ = mempty -- New ctor.
-
 instance AllVars (LHsDecl GhcPs) where
   allVars (L l (ValD _ bind)) = allVars (L l bind :: LHsBind GhcPs)
-
-  allVars _ = mempty  -- We only consider value bindings.
+  allVars _ = mempty
 
 
 vars :: FreeVars a => a -> [String]
