@@ -12,6 +12,7 @@ import System.Console.CmdArgs.Verbosity
 import GHC.Util.DynFlags
 import Data.List.Extra
 import GHC.Conc
+import System.Directory
 import System.Exit
 import System.IO.Extra
 import System.Time.Extra
@@ -23,14 +24,13 @@ import Config.Read
 import Config.Type
 import Config.Compute
 import Report
+import Summary
 import Idea
 import Apply
 import Test.All
 import Hint.All
-import Grep
 import Refact
 import Timing
-import Test.Proof
 import Parallel
 import GHC.All
 import CC
@@ -51,44 +51,26 @@ import EmbedData
 --   on your server with untrusted input.
 hlint :: [String] -> IO [Idea]
 hlint args = do
-    initGlobalDynFlags
+    startTimings
     cmd <- getCmd args
-    case cmd of
-        CmdMain{} -> do
-            startTimings
-            (time, xs) <- duration $ hlintMain args cmd
-            when (cmdTiming cmd) $ do
-                printTimings
-                putStrLn $ "Took " ++ showDuration time
-            pure $ if cmdNoExitCode cmd then [] else xs
-        CmdGrep{} -> hlintGrep cmd >> pure []
-        CmdTest{} -> hlintTest cmd >> pure []
+    timedIO "Initialise" "global flags" initGlobalDynFlags
+    if cmdTest cmd then
+        hlintTest cmd >> pure []
+     else do
+        (time, xs) <- duration $ hlintMain args cmd
+        when (cmdTiming cmd) $ do
+            printTimings
+            putStrLn $ "Took " ++ showDuration time
+        pure $ if cmdNoExitCode cmd then [] else xs
 
 hlintTest :: Cmd -> IO ()
-hlintTest cmd@CmdTest{..} =
-    if not $ null cmdProof then do
-        files <- cmdHintFiles cmd
-        s <- readFilesConfig files
-        let reps = if cmdReports == ["report.html"] then ["report.txt"] else cmdReports
-        mapM_ (proof reps s) cmdProof
-     else do
-        failed <- test cmd (\args -> do errs <- hlint args; unless (null errs) $ exitWith $ ExitFailure 1) cmdDataDir cmdGivenHints
-        when (failed > 0) exitFailure
+hlintTest cmd@CmdMain{..} = do
+    failed <- test cmd (\args -> do errs <- hlint args; unless (null errs) $ exitWith $ ExitFailure 1) cmdDataDir cmdGivenHints
+    when (failed > 0) exitFailure
 
 cmdParseFlags :: Cmd -> ParseFlags
 cmdParseFlags cmd = parseFlagsSetLanguage (cmdExtensions cmd) $ defaultParseFlags{cppFlags=cmdCpp cmd}
 
-hlintGrep :: Cmd -> IO ()
-hlintGrep cmd@CmdGrep{..} =
-    if null cmdFiles then
-        exitWithHelp
-     else do
-        files <- concatMapM (resolveFile cmd Nothing) cmdFiles
-        if null files then
-            errorIO "No files found"
-         else
-            runGrep cmdPattern (cmdParseFlags cmd) files
-
 withVerbosity :: Verbosity -> IO a -> IO a
 withVerbosity new act = do
     old <- getVerbosity
@@ -106,6 +88,12 @@ hlintMain args cmd@CmdMain{..}
                          ["- ignore: {name: " ++ show x ++ "}" | x <- bad]
             putStr $ unlines $ intercalate ["",""] $ group1:group2:groups
         pure []
+    | cmdGenerateSummary /= [] = do
+        forM_ cmdGenerateSummary $ \file -> timedIO "Summary" file $ do
+            whenNormal $ putStrLn $ "Writing summary to " ++ file ++ " ..."
+            summary <- generateSummary . snd =<< readAllSettings args cmd
+            writeFileBinary file summary
+        pure []
     | null cmdFiles && not (null cmdFindHints) = do
         hints <- concatMapM (resolveFile cmd Nothing) cmdFindHints
         mapM_ (putStrLn . fst <=< computeSettings (cmdParseFlags cmd)) hints >> pure []
@@ -123,11 +111,17 @@ runHlintMain args cmd tmpFile = do
 
 resolveFiles :: Cmd -> Maybe FilePath -> IO Cmd
 resolveFiles cmd@CmdMain{..} tmpFile = do
+    -- if the first file is named 'lint' and there is no 'lint' file
+    -- then someone is probably invoking the older hlint multi-mode command
+    -- so skip it
+    cmdFiles <- if not $ ["lint"] `isPrefixOf` cmdFiles then pure cmdFiles else do
+        b <- doesDirectoryExist "lint"
+        pure $ if b then cmdFiles else drop1 cmdFiles
+
     files <- concatMapM (resolveFile cmd tmpFile) cmdFiles
     if null files
         then error "No files found"
         else pure cmd { cmdFiles = files }
-resolveFiles cmd _ = pure cmd
 
 readAllSettings :: [String] -> Cmd -> IO (Cmd, [Setting])
 readAllSettings args1 cmd@CmdMain{..} = do
@@ -140,6 +134,8 @@ readAllSettings args1 cmd@CmdMain{..} = do
     cmd@CmdMain{..} <- if null args2 then pure cmd else getCmd $ args2 ++ args1 -- command line arguments are passed last
     settings2 <- concatMapM (fmap snd . computeSettings (cmdParseFlags cmd)) cmdFindHints
     let settings3 = [SettingClassify $ Classify Ignore x "" "" | x <- cmdIgnore]
+    cmdThreads <- if cmdThreads == 0 then getNumProcessors else pure cmdThreads
+    cmd <- pure CmdMain {..}
     pure (cmd, settings1 ++ settings2 ++ settings3)
     where
         enableGroup groupName =
@@ -150,9 +146,8 @@ readAllSettings args1 cmd@CmdMain{..} = do
             ]
 
 runHints :: [String] -> [Setting] -> Cmd -> IO [Idea]
-runHints args settings cmd@CmdMain{..} = do
-    j <- if cmdThreads == 0 then getNumProcessors else pure cmdThreads
-    withNumCapabilities j $ do
+runHints args settings cmd@CmdMain{..} =
+    withNumCapabilities cmdThreads $ do
         let outStrLn = whenNormal . putStrLn
         ideas <- getIdeas cmd settings
         ideas <- pure $ if cmdShowAll then ideas else  filter (\i -> ideaSeverity i /= Ignore) ideas
@@ -167,7 +162,7 @@ runHints args settings cmd@CmdMain{..} = do
             handleRefactoring ideas cmdFiles cmd
          else do
             usecolour <- cmdUseColour cmd
-            showItem <- if usecolour then showANSI else pure show
+            let showItem = if usecolour then showIdeaANSI else show
             mapM_ (outStrLn . showItem) ideas
             handleReporting ideas cmd
         pure ideas
