@@ -8,11 +8,11 @@ import Data.List.Extra
 import qualified Data.List.NonEmpty as NE
 import Data.Maybe
 import Data.Version
-import HSE.All
 import Timing
 import Paths_hlint
 import HsColour
 import EmbedData
+import qualified GHC.Util as GHC
 
 
 writeTemplate :: FilePath -> [(String,[String])] -> FilePath -> IO ()
@@ -28,7 +28,7 @@ writeReport dataDir file ideas = timedIO "Report" file $ writeTemplate dataDir i
     where
         generateIds :: [String] -> [(String,Int)] -- sorted by name
         generateIds = map (NE.head &&& length) . NE.group -- must be already sorted
-        files = generateIds $ sort $ map (srcSpanFilename . ideaSpan) ideas
+        files = generateIds $ sort $ map (GHC.srcSpanFilename . ideaSpan) ideas
         hints = generateIds $ map hintName $ sortOn (negate . fromEnum . ideaSeverity &&& hintName) ideas
         hintName x = show (ideaSeverity x) ++ ": " ++ ideaHint x
 
@@ -41,10 +41,10 @@ writeReport dataDir file ideas = timedIO "Report" file $ writeTemplate dataDir i
                          ("HINTS",list "hint" hints),("FILES",list "file" files)]
 
         content = concatMap (\i -> writeIdea (getClass i) i) ideas
-        getClass i = "hint" ++ f hints (hintName i) ++ " file" ++ f files (srcSpanFilename $ ideaSpan i)
+        getClass i = "hint" ++ f hints (hintName i) ++ " file" ++ f files (GHC.srcSpanFilename $ ideaSpan i)
             where f xs x = show $ fromJust $ findIndex ((==) x . fst) xs
 
-        list mode = zipWith f [0..]
+        list mode = zipWithFrom f 0
             where
                 f i (name,n) = "<li><a id=" ++ show id ++ " href=\"javascript:show('" ++ id ++ "')\">" ++
                                escapeHTML name ++ " (" ++ show n ++ ")</a></li>"
@@ -54,7 +54,7 @@ writeReport dataDir file ideas = timedIO "Report" file $ writeTemplate dataDir i
 writeIdea :: String -> Idea -> [String]
 writeIdea cls Idea{..} =
     ["<div class=" ++ show cls ++ ">"
-    ,escapeHTML (showSrcLoc (getPointLoc ideaSpan) ++ ": " ++ show ideaSeverity ++ ": " ++ ideaHint) ++ "<br/>"
+    ,escapeHTML (GHC.showSrcSpan ideaSpan ++ ": " ++ show ideaSeverity ++ ": " ++ ideaHint) ++ "<br/>"
     ,"Found<br/>"
     ,hsColourHTML ideaFrom] ++
     (case ideaTo of
